@@ -1,18 +1,17 @@
+import argparse
+import logging
 import os
 import sys
-import logging
-import argparse
 
-from n2y import notion
-from n2y.database import Database
 from n2y.blocks import LinkToPageBlock
+from n2y.database import Database
 from n2y.errors import UseNextClass
+from n2y.logger import HANDLER
+from n2y.logger import logger as log
+from n2y.mentions import PageMention
+from n2y.notion import Client
 from n2y.page import Page
 from n2y.utils import id_from_share_link
-from n2y.mentions import PageMention
-
-logger = None
-
 
 plugin_key = "audit"
 
@@ -24,11 +23,13 @@ class ReportingPageMention(PageMention):
         super().__init__(client, notion_data, plain_text, block)
         if plugin_key not in block.page.plugin_data:
             block.page.plugin_data[plugin_key] = []
-        block.page.plugin_data[plugin_key].append({
-            "block_url": block.notion_url,
-            "linked_page_id": self.notion_page_id,
-            "type": "page mention",
-        })
+        block.page.plugin_data[plugin_key].append(
+            {
+                "block_url": block.notion_url,
+                "linked_page_id": self.notion_page_id,
+                "type": "page mention",
+            }
+        )
 
 
 class ReportingLinkToPageBlock(LinkToPageBlock):
@@ -36,43 +37,49 @@ class ReportingLinkToPageBlock(LinkToPageBlock):
         super().__init__(client, notion_data, page, get_children)
         if plugin_key not in page.plugin_data:
             page.plugin_data[plugin_key] = []
-        page.plugin_data[plugin_key].append({
-            "block_url": self.notion_url,
-            "linked_page_id": self.linked_node_id,
-            "type": "link to page",
-        })
+        page.plugin_data[plugin_key].append(
+            {
+                "block_url": self.notion_url,
+                "linked_page_id": self.linked_node_id,
+                "type": "link to page",
+            }
+        )
 
 
 def cli_main():
     args = sys.argv[1:]
-    access_token = os.environ.get('NOTION_ACCESS_TOKEN', None)
+    access_token = os.environ.get("NOTION_ACCESS_TOKEN", None)
+
     sys.exit(main(args, access_token))
 
 
-def main(raw_args, access_token):
+def main(raw_args, access_token, logger=log):
     parser = argparse.ArgumentParser(
         description="Audit a set of Notion pages for external links",
         formatter_class=argparse.RawTextHelpFormatter,
     )
     parser.add_argument("object_id", help="The id or url for a Notion database or page")
     parser.add_argument(
-        "--verbosity", '-v', default='INFO',
+        "--verbosity",
+        "-v",
+        default="INFO",
         help="Level to set the root logging module to",
     )
     parser.add_argument(
-        "--logging-format", default='%(asctime)s - %(levelname)s: %(message)s',
+        "--logging-format",
+        default="%(asctime)s - %(levelname)s - n2y.%(module)s: %(message)s",
         help="Default format used when logging",
     )
 
     args = parser.parse_args(raw_args)
 
     logging_level = logging.__dict__[args.verbosity]
-    logging.basicConfig(format=args.logging_format, level=logging_level)
-    global logger
-    logger = logging.getLogger(__name__)
+    new_formatter = logging.Formatter(args.logging_format)
+    logger.setLevel(logging_level)
+    HANDLER.setFormatter(new_formatter)
 
     if access_token is None:
-        logger.critical('No NOTION_ACCESS_TOKEN environment variable is set')
+        logger.critical("No NOTION_ACCESS_TOKEN environment variable is set")
         return 1
 
     object_id = id_from_share_link(args.object_id)
@@ -93,7 +100,7 @@ def main(raw_args, access_token):
         },
     }
 
-    client = notion.Client(access_token)
+    client = Client(access_token, logger=logger)
     client.load_plugin(plugins)
 
     node = client.get_page_or_database(object_id)
@@ -103,7 +110,7 @@ def main(raw_args, access_token):
             "Unable to find database or page with id %s. "
             "Perhaps its not shared with the integration?"
         )
-        logger.error(msg, object_id)
+        client.logger.error(msg, object_id)
         return 2
 
     references = {}
@@ -119,7 +126,7 @@ def main(raw_args, access_token):
 def exclude_internal_references(references):
     external_references = {}
     for page_id, links in references.items():
-        external_links = [l for l in links if l['linked_page_id'] not in references]
+        external_links = [l for l in links if l["linked_page_id"] not in references]
         external_references[page_id] = external_links
     return external_references
 
@@ -142,13 +149,15 @@ def audit_node(node, references, depth):
 
 
 def audit_database(database, references, depth):
-    logger.info("%sDatabase %s", " " * depth, database.title.to_plain_text())
+    database.client.logger.info(
+        "%sDatabase %s", " " * depth, database.title.to_plain_text()
+    )
     for page in database.children:
         audit_page(page, references, depth + 1)
 
 
 def audit_page(page, references, depth):
-    logger.info("%sAuditing %s", " " * depth, page.title.to_plain_text())
+    page.client.logger.info("%sAuditing %s", " " * depth, page.title.to_plain_text())
     assert page.notion_id not in references  # expect that each page is visited once
     page.block  # load all of the blocks
     references[page.notion_id] = page.plugin_data.get(plugin_key, [])
