@@ -1,16 +1,40 @@
-import logging
 from itertools import groupby
+from re import match
 from urllib.parse import urljoin
 
 from pandoc.types import (
-    Str, Para, Plain, Header, CodeBlock, BulletList, OrderedList, Decimal,
-    Period, Meta, Pandoc, Link, HorizontalRule, BlockQuote, Image, MetaString,
-    Table, TableHead, TableBody, TableFoot, RowHeadColumns, Row, Cell, RowSpan,
-    ColSpan, ColWidthDefault, AlignDefault, Caption, Math, DisplayMath,
+    AlignDefault,
+    BlockQuote,
+    BulletList,
+    Caption,
+    Cell,
+    CodeBlock,
+    ColSpan,
+    ColWidthDefault,
+    Decimal,
+    DisplayMath,
+    Header,
+    HorizontalRule,
+    Image,
+    Link,
+    Math,
+    OrderedList,
+    Pandoc,
+    Para,
+    Period,
+    Plain,
+    Row,
+    RowHeadColumns,
+    RowSpan,
+    Str,
+    Table,
+    TableBody,
+    TableFoot,
+    TableHead,
 )
 
-
-logger = logging.getLogger(__name__)
+from n2y.notion_mocks import mock_block, mock_rich_text_array
+from n2y.utils import header_id_from_text, pandoc_write_or_log_errors, yaml_map_to_meta
 
 
 class Block:
@@ -35,20 +59,19 @@ class Block:
         2. In some cases a block may choose not to get child blocks.
            Currently, all blocks load all children.
         """
-        logger.debug('Instantiating "%s" block', type(self).__name__)
+        client.logger.debug('Instantiating "%s" block', type(self).__name__)
         self.client = client
         self.page = page
 
-        self.notion_id = notion_data['id']
-        self.created_time = notion_data['created_time']
-        self.created_by = notion_data['created_by']
-        self.last_edited_time = notion_data['last_edited_time']
-        self.last_edited_by = notion_data['last_edited_by']
-        self.has_children = notion_data['has_children']
-        self.archived = notion_data['archived']
-        self.notion_type = notion_data['type']
+        self.notion_id = notion_data["id"]
+        self.created_time = notion_data["created_time"]
+        self.created_by = notion_data["created_by"]
+        self.last_edited_time = notion_data["last_edited_time"]
+        self.last_edited_by = notion_data["last_edited_by"]
+        self.has_children = notion_data["has_children"]
+        self.archived = notion_data["archived"]
+        self.notion_type = notion_data["type"]
         self.notion_data = notion_data
-
         if get_children:
             self.get_children()
         else:
@@ -64,7 +87,6 @@ class Block:
         raise NotImplementedError()
 
     def children_to_pandoc(self):
-        assert self.has_children
         pandoc_ast = []
         for block_type, blocks in groupby(self.children, lambda c: type(c)):
             if issubclass(block_type, ListItemBlock):
@@ -72,7 +94,9 @@ class Block:
             else:
                 for b in blocks:
                     result = b.to_pandoc()
-                    if isinstance(result, list):
+                    if block_type == ChildPageBlock:
+                        pandoc_ast.extend(result[1])
+                    elif isinstance(result, list):
                         # a few blocks return lists of nodes
                         pandoc_ast.extend(result)
                     elif result is not None:
@@ -89,7 +113,7 @@ class Block:
     @property
     def notion_url(self):
         # the notion URL's don't work if the dashes from the block ID are present
-        fragment = '#' + self.notion_id.replace('-', '')
+        fragment = "#" + self.notion_id.replace("-", "")
         if self.page is None:
             return fragment
         else:
@@ -98,7 +122,7 @@ class Block:
 
 class ListItemBlock(Block):
     @classmethod
-    def list_to_pandoc(klass, items):
+    def list_to_pandoc(cls, items):
         raise NotImplementedError()
 
 
@@ -109,11 +133,13 @@ class ChildPageBlock(Block):
 
     def to_pandoc(self):
         assert self.children is not None
-        if self.children:
-            children = self.children_to_pandoc()
-            return Pandoc(Meta({'title': MetaString(self.title)}), children)
+        children = self.children_to_pandoc()
+        if self.page:
+            properties = self.page.properties_to_values()
         else:
-            return None
+            properties = {}
+        meta = yaml_map_to_meta(properties)
+        return Pandoc(meta, children)
 
 
 class EquationBlock(Block):
@@ -158,41 +184,138 @@ class BulletedListItemBlock(ListItemBlock):
         content = [Plain(self.rich_text.to_pandoc())]
         if self.has_children:
             children = self.children_to_pandoc()
-            content.extend(children)
+            for child in children:
+                if isinstance(child, Table):
+                    # A bug in pandoc's markdown reader makes it unable to register mid-list tables.
+                    # This adds an extra space between the table and the previous list item, which
+                    # Allows the table to be read as such.
+                    content.append(Para([]))
+                content.append(child)
         return content
 
     @classmethod
-    def list_to_pandoc(klass, items):
+    def list_to_pandoc(cls, items):
         return BulletList([b.to_pandoc() for b in items])
 
 
 class ToDoListItemBlock(BulletedListItemBlock):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
-        self.checked = self.notion_type_data['checked']
+        self.checked = self.notion_type_data["checked"]
 
-        # TODO: Move this into the "to_pandoc" stage
-        box = '☒ ' if self.checked else '☐ '
+        box = "☒ " if self.checked else "☐ "
         self.rich_text.prepend(box)
 
 
-class NumberedListItemBlock(ListItemBlock):
+class NumberedListItemBlock(BulletedListItemBlock):
+    @classmethod
+    def list_to_pandoc(cls, items):
+        return OrderedList((1, Decimal(), Period()), [b.to_pandoc() for b in items])
+
+
+class TableOfContentsBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
-        super().__init__(client, notion_data, page, get_children)
-        self.rich_text = client.wrap_notion_rich_text_array(
-            self.notion_type_data["rich_text"], self
+        self.subheaders: list[Header] | None = notion_data[notion_data["type"]].get(
+            "subheaders", None
         )
+        super().__init__(client, notion_data, page, get_children)
+
+    def get_children(self):
+        if self.subheaders is not None:
+            children: list[TableOfContentsItemBlock] = []
+            subsections: list[list[Header]] = []
+            # Sometimes, the first header is not an H1, so we need to find the first
+            base: int = self.subheaders[0][0]
+            index: int = -1
+            for header in self.subheaders:
+                if header[0] == base:
+                    index += 1
+                    subsections.append([header])
+                if header[0] > base:
+                    subsections[index].append(header)
+                if header[0] < base:
+                    self.client.logger.warning(
+                        f'Skipping out-of-order header "{header[1][0]}" in table of'
+                        " contents for page named"
+                        f" {self.page.title.to_plain_text()} ({self.page.notion_url})."
+                        " The base header is an H{base} so all following headers"
+                        " should be H{base} or greater."
+                    )
+            for subsection in subsections:
+                notion_data = self.generate_item_block(subsection)
+                children.append(
+                    TableOfContentsItemBlock(self.client, notion_data, self.page)
+                )
+            if children:
+                self.has_children = True
+            self.children = children
+        else:
+            self.children = None
+
+    def get_subheaders(self, ast_list):
+        self.subheaders: list[Header] | None = []
+        for block in ast_list:
+            if isinstance(block, Header):
+                self.subheaders.append(block)
 
     def to_pandoc(self):
-        content = [Plain(self.rich_text.to_pandoc())]
-        if self.has_children:
-            children = self.children_to_pandoc()
-            content.extend(children)
-        return content
+        if not self.children:
+            return None
+        else:
+            return self.children_to_pandoc()
 
-    @classmethod
-    def list_to_pandoc(klass, items):
-        return OrderedList((1, Decimal(), Period()), [b.to_pandoc() for b in items])
+    def generate_item_block(self, section: list[Header]):
+        header = section.pop(0)
+        header_text = pandoc_write_or_log_errors(
+            header[2], "gfm", [], self.client.logger
+        )[:-1]
+        rich_text = mock_rich_text_array([(header_text, None, f"#{header[1][0]}")])
+        type_data = {
+            "header": header,
+            "subheaders": section,
+            "rich_text": rich_text,
+        }
+        return mock_block("table_of_contents_item", type_data)
+
+    def render_toc(self, ast_list):
+        self.get_subheaders(ast_list)
+        self.get_children()
+
+
+class TableOfContentsItemBlock(NumberedListItemBlock, TableOfContentsBlock):
+    def __init__(self, client, notion_data, page, get_children=True):
+        type_data = notion_data[notion_data["type"]]
+        self.header = type_data["header"]
+        self.level = self.header[0]
+        super().__init__(client, notion_data, page, get_children)
+
+    def get_children(self):
+        children: list[TableOfContentsItemBlock] = []
+        subsections: list[list[Header]] = []
+        index: int = -1
+        for header in self.subheaders:
+            if header[0] == self.level + 1:
+                index += 1
+                subsections.append([header])
+            if header[0] > self.level + 1:
+                try:
+                    subsections[index].append(header)
+                except IndexError:
+                    self.client.logger.warning(
+                        f'Skipping out-of-order header "{header[1][0]}" in table of'
+                        " contents for page named"
+                        f" {self.page.title.to_plain_text()} ({self.page.notion_url})."
+                        " Please create headers in sequence (i.e. proper document"
+                        f" formatting dictates that an H{self.level} should not be"
+                        f" immediately followed by an H{header[0]}, but an"
+                        f" H{self.level + 1})."
+                    )
+        for subsection in subsections:
+            notion_data = self.generate_item_block(subsection)
+            children.append(TableOfContentsItemBlock(self.client, notion_data, self.page))
+        if children:
+            self.has_children = True
+        self.children = children
 
 
 class HeadingBlock(Block):
@@ -209,7 +332,8 @@ class HeadingBlock(Block):
             rich_text.bold = False
 
     def to_pandoc(self):
-        return Header(self.level, ('', [], []), self.rich_text.to_pandoc())
+        section_id = header_id_from_text(self.rich_text.to_plain_text())
+        return Header(self.level, (section_id, [], []), self.rich_text.to_pandoc())
 
 
 class HeadingOneBlock(HeadingBlock):
@@ -233,50 +357,179 @@ class BookmarkBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
         self.url = self.notion_type_data["url"]
-        self.caption = client.wrap_notion_rich_text_array(self.notion_type_data["caption"], self)
+        self.caption = client.wrap_notion_rich_text_array(
+            self.notion_type_data["caption"], self
+        )
 
     def to_pandoc(self):
         if self.caption:
             caption_ast = self.caption.to_pandoc()
         else:
             caption_ast = [Str(self.url)]
-        return Para([Link(('', [], []), caption_ast, (self.url, ''))])
+        return Para([Link(("", [], []), caption_ast, (self.url, ""))])
 
 
 class FencedCodeBlock(Block):
     pandoc_highlight_languages = [
-        "abc", "actionscript", "ada", "agda", "apache", "asn1", "asp", "ats", "awk",
-        "bash", "bibtex", "boo", "c", "changelog", "clojure", "cmake", "coffee",
-        "coldfusion", "comments", "commonlisp", "cpp", "cs", "css", "curry", "d",
-        "default", "diff", "djangotemplate", "dockerfile", "dot", "doxygen",
-        "doxygenlua", "dtd", "eiffel", "elixir", "elm", "email", "erlang", "fasm",
-        "fortranfixed", "fortranfree", "fsharp", "gcc", "glsl", "gnuassembler", "go",
-        "graphql", "groovy", "hamlet", "haskell", "haxe", "html", "idris", "ini",
-        "isocpp", "j", "java", "javadoc", "javascript", "javascriptreact", "json",
-        "jsp", "julia", "kotlin", "latex", "lex", "lilypond", "literatecurry",
-        "literatehaskell", "llvm", "lua", "m4", "makefile", "mandoc", "markdown",
-        "mathematica", "matlab", "maxima", "mediawiki", "metafont", "mips", "modelines",
-        "modula2", "modula3", "monobasic", "mustache", "nasm", "nim", "noweb",
-        "objectivec", "objectivecpp", "ocaml", "octave", "opencl", "orgmode", "pascal",
-        "perl", "php", "pike", "postscript", "povray", "powershell", "prolog",
-        "protobuf", "pure", "purebasic", "python", "qml", "r", "raku", "relaxng",
-        "relaxngcompact", "rest", "rhtml", "roff", "ruby", "rust", "sass", "scala",
-        "scheme", "sci", "scss", "sed", "sgml", "sml", "spdxcomments", "sql",
-        "sqlmysql", "sqlpostgresql", "stan", "stata", "swift", "systemverilog", "tcl",
-        "tcsh", "texinfo", "toml", "typescript", "verilog", "vhdl", "xml", "xorg",
-        "xslt", "xul", "yacc", "yaml", "zsh",
+        "abc",
+        "actionscript",
+        "ada",
+        "agda",
+        "apache",
+        "asn1",
+        "asp",
+        "ats",
+        "awk",
+        "bash",
+        "bibtex",
+        "boo",
+        "c",
+        "changelog",
+        "clojure",
+        "cmake",
+        "coffee",
+        "coldfusion",
+        "comments",
+        "commonlisp",
+        "cpp",
+        "cs",
+        "css",
+        "curry",
+        "d",
+        "default",
+        "diff",
+        "djangotemplate",
+        "dockerfile",
+        "dot",
+        "doxygen",
+        "doxygenlua",
+        "dtd",
+        "eiffel",
+        "elixir",
+        "elm",
+        "email",
+        "erlang",
+        "fasm",
+        "fortranfixed",
+        "fortranfree",
+        "fsharp",
+        "gcc",
+        "glsl",
+        "gnuassembler",
+        "go",
+        "graphql",
+        "groovy",
+        "hamlet",
+        "haskell",
+        "haxe",
+        "html",
+        "idris",
+        "ini",
+        "isocpp",
+        "j",
+        "java",
+        "javadoc",
+        "javascript",
+        "javascriptreact",
+        "json",
+        "jsp",
+        "julia",
+        "kotlin",
+        "latex",
+        "lex",
+        "lilypond",
+        "literatecurry",
+        "literatehaskell",
+        "llvm",
+        "lua",
+        "m4",
+        "makefile",
+        "mandoc",
+        "markdown",
+        "mathematica",
+        "matlab",
+        "maxima",
+        "mediawiki",
+        "metafont",
+        "mips",
+        "modelines",
+        "modula2",
+        "modula3",
+        "monobasic",
+        "mustache",
+        "nasm",
+        "nim",
+        "noweb",
+        "objectivec",
+        "objectivecpp",
+        "ocaml",
+        "octave",
+        "opencl",
+        "orgmode",
+        "pascal",
+        "perl",
+        "php",
+        "pike",
+        "postscript",
+        "povray",
+        "powershell",
+        "prolog",
+        "protobuf",
+        "pure",
+        "purebasic",
+        "python",
+        "qml",
+        "r",
+        "raku",
+        "relaxng",
+        "relaxngcompact",
+        "rest",
+        "rhtml",
+        "roff",
+        "ruby",
+        "rust",
+        "sass",
+        "scala",
+        "scheme",
+        "sci",
+        "scss",
+        "sed",
+        "sgml",
+        "sml",
+        "spdxcomments",
+        "sql",
+        "sqlmysql",
+        "sqlpostgresql",
+        "stan",
+        "stata",
+        "swift",
+        "systemverilog",
+        "tcl",
+        "tcsh",
+        "texinfo",
+        "toml",
+        "typescript",
+        "verilog",
+        "vhdl",
+        "xml",
+        "xorg",
+        "xslt",
+        "xul",
+        "yacc",
+        "yaml",
+        "zsh",
     ]
 
     # TODO: finish filling in this mapping from Notion language names to
     # pandoc's supported language names
     notion_to_pandoc_highlight_languages = {
-        'c#': 'cs',
-        'c++': 'cpp',
-        'f#': 'fsharp',
-        'objective-c': 'objectivec',
-        'docker': 'dockerfile',
-        'coffee': 'coffeescript',
-        'shell': 'bash',  # seems better than nothing
+        "c#": "cs",
+        "c++": "cpp",
+        "f#": "fsharp",
+        "objective-c": "objectivec",
+        "docker": "dockerfile",
+        "coffee": "coffeescript",
+        "shell": "bash",  # seems better than nothing
     }
 
     def __init__(self, client, notion_data, page, get_children=True):
@@ -291,16 +544,17 @@ class FencedCodeBlock(Block):
 
     def to_pandoc(self):
         pandoc_language = self.notion_to_pandoc_highlight_languages.get(
-            self.language, self.language,
+            self.language,
+            self.language,
         )
         if pandoc_language not in self.pandoc_highlight_languages:
             if pandoc_language != "plain text":
                 msg = 'Dropping syntax highlighting for unsupported language "%s" (%s)'
-                logger.warning(msg, pandoc_language, self.notion_url)
+                self.client.logger.warning(msg, pandoc_language, self.notion_url)
             language = []
         else:
             language = [pandoc_language]
-        return CodeBlock(('', language, []), self.rich_text.to_plain_text())
+        return CodeBlock(("", language, []), self.rich_text.to_plain_text())
 
 
 class QuoteBlock(ParagraphBlock):
@@ -322,8 +576,19 @@ class QuoteBlock(ParagraphBlock):
 class FileBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
-        self.file = client.wrap_notion_file(notion_data['file'])
-        self.caption = client.wrap_notion_rich_text_array(self.notion_type_data["caption"], self)
+        self.file = client.wrap_notion_file(notion_data["file"])
+        self.caption = client.wrap_notion_rich_text_array(
+            self.notion_type_data["caption"], self
+        )
+        potential_name = match(
+            (
+                r".+(?:[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/)+"
+                r"(?P<name>.+)\?.+"
+            ),
+            self.file.url,
+        )
+        name = potential_name.groups("name") if potential_name else None
+        self.name = name[0] if name else None
 
     def to_pandoc(self):
         url = None
@@ -331,7 +596,7 @@ class FileBlock(Block):
             url = self.file.url
         elif self.file.type == "file":
             url = self.client.download_file(self.file.url, self.page, self.notion_id)
-        content_ast = [Link(('', [], []), [Str(url)], (url, ''))]
+        content_ast = [Link(("", [], []), [Str(self.name or url)], (url, ""))]
         if self.caption:
             caption_ast = self.caption.to_pandoc()
             return render_with_caption(content_ast, caption_ast)
@@ -341,8 +606,10 @@ class FileBlock(Block):
 class ImageBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
-        self.file = client.wrap_notion_file(notion_data['image'])
-        self.caption = client.wrap_notion_rich_text_array(self.notion_type_data["caption"], self)
+        self.file = client.wrap_notion_file(notion_data["image"])
+        self.caption = client.wrap_notion_rich_text_array(
+            self.notion_type_data["caption"], self
+        )
 
     def to_pandoc(self):
         url = None
@@ -350,18 +617,20 @@ class ImageBlock(Block):
             url = self.file.url
         elif self.file.type == "file":
             url = self.client.download_file(self.file.url, self.page, self.notion_id)
-        img_alt = [Str(url)]
+        caption = []
+        fig_flag = ""
         if self.caption:
-            img_alt = self.caption.to_pandoc()
-        return Para([Image(('', [], []), img_alt, (url, ''))])
+            fig_flag = "fig:"
+            caption = self.caption.to_pandoc()
+        return Para([Image(("", [], []), caption, (url, fig_flag))])
 
 
 class TableBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
-        self.has_column_header = self.notion_type_data['has_column_header']
-        self.has_row_header = self.notion_type_data['has_row_header']
-        self.table_width = self.notion_type_data['table_width']
+        self.has_column_header = self.notion_type_data["has_column_header"]
+        self.has_row_header = self.notion_type_data["has_row_header"]
+        self.table_width = self.notion_type_data["table_width"]
 
     def to_pandoc(self):
         children = self.children_to_pandoc()
@@ -377,17 +646,12 @@ class TableBlock(Block):
         # Notion does not have cell alignment or width options, sticking with defaults.
         colspec = [(AlignDefault(), ColWidthDefault()) for _ in range(self.table_width)]
         table = Table(
-            ('', [], []),
+            ("", [], []),
             Caption(None, []),
             colspec,
-            TableHead(('', [], []), header_rows),
-            [TableBody(
-                ('', [], []),
-                RowHeadColumns(row_header_columns),
-                [],
-                children
-            )],
-            TableFoot(('', [], []), [])
+            TableHead(("", [], []), header_rows),
+            [TableBody(("", [], []), RowHeadColumns(row_header_columns), [], children)],
+            TableFoot(("", [], []), []),
         )
         return table
 
@@ -396,19 +660,24 @@ class RowBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
         self.cells = [
-            client.wrap_notion_rich_text_array(nc, self, True)
+            client.wrap_notion_rich_text_array(nc, self)
             for nc in self.notion_type_data["cells"]
         ]
 
     def to_pandoc(self):
-        cells = [Cell(
-            ('', [], []),
-            AlignDefault(),
-            RowSpan(1),
-            ColSpan(1),
-            [Plain(cell.to_pandoc())]
-        ) for cell in self.cells]
-        return Row(('', [], []), cells)
+        cells = []
+        for cell in self.cells:
+            pandoc = cell.to_pandoc()
+            cells.append(
+                Cell(
+                    ("", [], []),
+                    AlignDefault(),
+                    RowSpan(1),
+                    ColSpan(1),
+                    [Plain(pandoc)],
+                )
+            )
+        return Row(("", [], []), cells)
 
 
 class ColumnListBlock(Block):
@@ -416,33 +685,15 @@ class ColumnListBlock(Block):
         super().__init__(client, notion_data, page, get_children)
 
     def to_pandoc(self):
-        cells = []
+        pandoc = []
         if self.children:
-            cells = self.children_to_pandoc()
-        colspec = [(AlignDefault(), ColWidthDefault()) for _ in range(len(cells))]
-        table = Table(
-            ('', [], []),
-            Caption(None, []),
-            colspec,
-            TableHead(('', [], []), []),
-            [TableBody(
-                ('', [], []),
-                RowHeadColumns(0), [],
-                [Row(('', [], []), cells)])],
-            TableFoot(('', [], []), [])
-        )
-        return table
+            pandoc = self.children_to_pandoc()
+        return pandoc
 
 
 class ColumnBlock(Block):
     def to_pandoc(self):
-        return Cell(
-            ('', [], []),
-            AlignDefault(),
-            RowSpan(1),
-            ColSpan(1),
-            self.children_to_pandoc()
-        )
+        return self.children_to_pandoc()
 
 
 class ToggleBlock(Block):
@@ -493,10 +744,6 @@ class NoopBlock(Block):
         return None
 
 
-class TableOfContentsBlock(NoopBlock):
-    pass
-
-
 class BreadcrumbBlock(NoopBlock):
     pass
 
@@ -511,29 +758,32 @@ class TemplateBlock(NoopBlock):
 
 class WarningBlock(NoopBlock):
     def to_pandoc(self):
-        logger.warning('Skipping unsupported "%s" block (%s)', self.notion_type, self.notion_url)
+        self.client.logger.warning(
+            'Skipping unsupported "%s" block (%s)', self.notion_type, self.notion_url
+        )
         return None
 
 
 class ChildDatabaseBlock(NoopBlock):
-    def to_pandoc(self):
-        msg = (
-            'Skipping unsupported "%s" block (%s). '
-            'Perhaps you can convert the database into a simple table?'
-        )
-        logger.warning(msg, self.notion_type, self.notion_url)
-        return None
+    pass
 
 
 class EmbedBlock(WarningBlock):
     pass
 
 
-class VideoBlock(Block):
+class ContentBlock(Block):
+    """
+    Generic base class for blocks that contain file-based content that should be
+    downloaded and then linked to.
+    """
+
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
-        self.file = client.wrap_notion_file(notion_data['video'])
-        self.caption = client.wrap_notion_rich_text_array(self.notion_type_data["caption"], self)
+        self.file = client.wrap_notion_file(self.notion_type_data)
+        self.caption = client.wrap_notion_rich_text_array(
+            self.notion_type_data["caption"], self
+        )
 
     def to_pandoc(self):
         url = None
@@ -541,22 +791,33 @@ class VideoBlock(Block):
             url = self.file.url
         elif self.file.type == "file":
             url = self.client.download_file(self.file.url, self.page, self.notion_id)
-        content_ast = [Link(('', [], []), [Str(url)], (url, ''))]
+        content_ast = [Link(("", [], []), [Str(url)], (url, ""))]
         if self.caption:
             caption_ast = self.caption.to_pandoc()
             return render_with_caption(content_ast, caption_ast)
-        return Para(content_ast)
+        else:
+            return Para(content_ast)
+
+
+class AudioBlock(ContentBlock):
+    pass
+
+
+class VideoBlock(ContentBlock):
+    pass
 
 
 class PdfBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         super().__init__(client, notion_data, page, get_children)
-        self.pdf = client.wrap_notion_file(notion_data['pdf'])
-        self.caption = client.wrap_notion_rich_text_array(self.notion_type_data["caption"], self)
+        self.pdf = client.wrap_notion_file(notion_data["pdf"])
+        self.caption = client.wrap_notion_rich_text_array(
+            self.notion_type_data["caption"], self
+        )
 
     def to_pandoc(self):
         url = self.client.download_file(self.pdf.url, self.page, self.notion_id)
-        content_ast = [Link(('', [], []), [Str(url)], (url, ''))]
+        content_ast = [Link(("", [], []), [Str(url)], (url, ""))]
         if self.caption:
             caption_ast = self.caption.to_pandoc()
             return render_with_caption(content_ast, caption_ast)
@@ -580,6 +841,7 @@ class SyncedBlock(Block):
     def __init__(self, client, notion_data, page, get_children=True):
         self.original = notion_data[notion_data["type"]]["synced_from"] is None
         super().__init__(client, notion_data, page, get_children=self.original)
+        self.is_recursive = None
         # Synced blocks will always have children unless not shared
         # (There will always be at least one UnsupportedBlock child)
         self.shared = self.has_children
@@ -587,15 +849,33 @@ class SyncedBlock(Block):
 
     def _get_synced_block_children(self):
         if not self.original and self.shared:
-            return self.client.get_child_blocks(
-                self.notion_type_data["synced_from"]["block_id"],
-                self.page, True,
+            # This last condition is to protect against recursive synced blocks while
+            # still allowing synced blocks that are children of other synced blocks
+            # (once Notion confirms that this bug has been addressed it can be removed)
+            parent = self.notion_data.get("parent", None)
+            self.is_recursive = (
+                parent
+                and self.notion_type_data["synced_from"]["block_id"]
+                == parent[parent["type"]]
             )
+            if not self.is_recursive:
+                return self.client.get_child_blocks(
+                    self.notion_type_data["synced_from"]["block_id"],
+                    self.page,
+                    True,
+                )
         return self.children
 
     def to_pandoc(self):
         if not self.shared:
-            logger.warning('Skipping un-shared synced block (%s)', self.notion_url)
+            self.client.logger.warning(
+                "Skipping un-shared synced block (%s)", self.notion_url
+            )
+            return None
+        elif self.is_recursive:
+            self.client.logger.warning(
+                "Skipping recursive synced block (%s)", self.notion_url
+            )
             return None
         return self.children_to_pandoc()
 
@@ -620,8 +900,8 @@ class LinkToPageBlock(Block):
             raise NotImplementedError(f"Unknown link type: {self.link_type}")
 
         if node is None:
-            msg = "Permission denied when attempting to access linked node [%s]"
-            logger.warning(msg, self.notion_url)
+            msg = "Permission denied when attempting to access linked node (%r)"
+            self.client.logger.warning(msg, self.notion_url)
             return None
         else:
             title = node.title.to_pandoc()
@@ -629,16 +909,28 @@ class LinkToPageBlock(Block):
 
 
 def render_with_caption(content_ast, caption_ast):
-    header_cell_args = [('', [], []), AlignDefault(), RowSpan(1), ColSpan(1), [Plain(content_ast)]]
-    body_cell_args = [('', [], []), AlignDefault(), RowSpan(1), ColSpan(1), [Plain(caption_ast)]]
-    body_row = Row(('', [], []), [Cell(*body_cell_args)])
+    header_cell_args = [
+        ("", [], []),
+        AlignDefault(),
+        RowSpan(1),
+        ColSpan(1),
+        [Plain(content_ast)],
+    ]
+    body_cell_args = [
+        ("", [], []),
+        AlignDefault(),
+        RowSpan(1),
+        ColSpan(1),
+        [Plain(caption_ast)],
+    ]
+    body_row = Row(("", [], []), [Cell(*body_cell_args)])
     return Table(
-        ('', [], []),
+        ("", [], []),
         Caption(None, []),
         [(AlignDefault(), ColWidthDefault())],
-        TableHead(('', [], []), [Row(('', [], []), [Cell(*header_cell_args)])]),
-        [TableBody(('', [], []), RowHeadColumns(0), [], [body_row])],
-        TableFoot(('', [], []), [])
+        TableHead(("", [], []), [Row(("", [], []), [Cell(*header_cell_args)])]),
+        [TableBody(("", [], []), RowHeadColumns(0), [], [body_row])],
+        TableFoot(("", [], []), []),
     )
 
 
@@ -655,6 +947,7 @@ DEFAULT_BLOCKS = {
     "child_database": ChildDatabaseBlock,
     "embed": EmbedBlock,
     "image": ImageBlock,
+    "audio": AudioBlock,
     "video": VideoBlock,
     "file": FileBlock,
     "pdf": PdfBlock,
@@ -664,7 +957,8 @@ DEFAULT_BLOCKS = {
     "equation": EquationBlock,
     "divider": DividerBlock,
     "table_of_contents": TableOfContentsBlock,
-    "breadcrumb": TableOfContentsBlock,
+    "table_of_contents_item": TableOfContentsItemBlock,
+    "breadcrumb": BreadcrumbBlock,
     "column": ColumnBlock,
     "column_list": ColumnListBlock,
     "link_preview": LinkPreviewBlock,
