@@ -1,9 +1,3 @@
-import logging
-
-
-logger = logging.getLogger(__name__)
-
-
 class Emoji:
     """
     See https://developers.notion.com/reference/emoji-object
@@ -11,4 +5,4 @@ class Emoji:
 
     def __init__(self, client, notion_data):
         self.client = client
-        self.emoji = notion_data['emoji']
+        self.emoji = notion_data["emoji"]
