@@ -1,40 +1,33 @@
-import logging
-
-from .blocks import ChildDatabaseBlock, ChildPageBlock
-
-from n2y.utils import fromisoformat
+from n2y.blocks import ChildDatabaseBlock, ChildPageBlock, TableOfContentsBlock
 from n2y.property_values import TitlePropertyValue
-
-
-logger = logging.getLogger(__name__)
+from n2y.utils import fromisoformat
 
 
 class Page:
     def __init__(self, client, notion_data):
-        logger.debug("Instantiating page")
-        self.client = client
-
+        client.logger.debug("Instantiating page")
+        self.notion_parent = notion_data["parent"]
+        self.archived = notion_data["archived"]
+        self.notion_url = notion_data["url"]
+        self.notion_id = notion_data["id"]
         self.notion_data = notion_data
-        self.notion_id = notion_data['id']
-        self.created_time = fromisoformat(notion_data['created_time'])
-        self.created_by = client.wrap_notion_user(notion_data['created_by'])
-        self.last_edited_time = fromisoformat(notion_data['last_edited_time'])
-        self.last_edited_by = client.wrap_notion_user(notion_data['last_edited_by'])
-        self.archived = notion_data['archived']
-        self.emoji = self._init_icon(notion_data['icon'])
-        self.cover = notion_data['cover'] and client.wrap_notion_file(notion_data['cover'])
-        self.archived = notion_data['archived']
-        self.properties = {
-            k: client.wrap_notion_property_value(npv)
-            for k, npv in notion_data['properties'].items()
-        }
-        self.notion_parent = notion_data['parent']
-        self.notion_url = notion_data['url']
-
-        self._block = None
-        self._children = None
-
         self.plugin_data = {}
+        self._children = None
+        self.client = client
+        self._block = None
+
+        self.created_time = fromisoformat(notion_data["created_time"])
+        self.last_edited_time = fromisoformat(notion_data["last_edited_time"])
+        self.created_by = client.wrap_notion_user(notion_data["created_by"])
+        self.last_edited_by = client.wrap_notion_user(notion_data["last_edited_by"])
+        self.icon = self._init_icon(notion_data["icon"])
+        self.cover = notion_data["cover"] and client.wrap_notion_file(
+            notion_data["cover"]
+        )
+        self.properties = {
+            k: client.wrap_notion_property_value(npv, self)
+            for k, npv in notion_data["properties"].items()
+        }
 
     def _init_icon(self, icon_notion_data):
         """
@@ -100,8 +93,31 @@ class Page:
             assert parent_type == "database_id"
             return self.client.get_database(self.notion_parent["database_id"])
 
-    def to_pandoc(self):
-        return self.block.to_pandoc()
+    def generate_toc(self, ast):
+        """
+        Generate a table of contents for the page.
+        """
+        toc_indecies = []
 
-    def properties_to_values(self):
-        return {k: v.to_value() for k, v in self.properties.items()}
+        def is_toc(i, block):
+            if type(block) is TableOfContentsBlock:
+                toc_indecies.append(i)
+                return True
+            return False
+
+        if any(is_toc(*t) for t in enumerate(self.block.children)):
+            child_ast = ast[1]
+            for i in toc_indecies:
+                self.block.children[i].render_toc(child_ast)
+            ast = self.block.to_pandoc()
+        return ast
+
+    def to_pandoc(self, ignore_toc=False):
+        ast = self.block.to_pandoc()
+        return ast if ignore_toc else self.generate_toc(ast)
+
+    def properties_to_values(self, pandoc_format=None, pandoc_options=None):
+        return {
+            k: v.to_value(pandoc_format, pandoc_options)
+            for k, v in self.properties.items()
+        }
