@@ -1,9 +1,5 @@
-import logging
-
 from n2y.blocks import LinkToPageBlock
 
-logger = logging.getLogger(__name__)
-
 
 class ExpandingLinkToPageBlock(LinkToPageBlock):
     """
@@ -23,9 +19,10 @@ class ExpandingLinkToPageBlock(LinkToPageBlock):
             return page.block.children_to_pandoc()
         else:
             # TODO: Might be expanded to handle links to databases as well.
-            logger.warning(
-                'Links to databases (to:%s from:%s) not supported at this time.',
-                self.linked_node_id, self.page.notion_id
+            self.client.logger.warning(
+                "Links to databases (to:%s from:%s) not supported at this time.",
+                self.linked_node_id,
+                self.page.notion_id,
             )
             return None
 
