@@ -1,17 +1,13 @@
 import re
-import logging
 
-from pandoc.types import Note, Str, Para
+from pandoc.types import Note, Para, Str
 
-from n2y.rich_text import TextRichText
 from n2y.blocks import ParagraphBlock
 from n2y.errors import UseNextClass
-
+from n2y.rich_text import TextRichText
 
 plugin_data_key = "n2y.plugins.footnotes"
 
-logger = logging.getLogger(__name__)
-
 
 class ParagraphWithFootnoteBlock(ParagraphBlock):
     def __init__(self, client, notion_data, page, get_children=True):
@@ -28,13 +24,15 @@ class ParagraphWithFootnoteBlock(ParagraphBlock):
         if plugin_data_key not in self.page.plugin_data:
             self.page.plugin_data[plugin_data_key] = {}
         if self._footnote() not in self.page.plugin_data[plugin_data_key]:
-            self.page.plugin_data[plugin_data_key][self._footnote()] = self._footnote_ast()
+            self.page.plugin_data[plugin_data_key][
+                self._footnote()
+            ] = self._footnote_ast()
             if self._footnote_empty():
                 msg = 'Empty footnote "[%s]" (%s)'
-                logger.warning(msg, self._footnote(), self.notion_url)
+                self.client.logger.warning(msg, self._footnote(), self.notion_url)
         else:
             msg = 'Multiple footnotes for "[%s]", skipping latest (%s)'
-            logger.warning(msg, self._footnote(), self.notion_url)
+            self.client.logger.warning(msg, self._footnote(), self.notion_url)
 
     def _is_footnote(self):
         return self._footnote() is not None
@@ -61,8 +59,8 @@ class ParagraphWithFootnoteBlock(ParagraphBlock):
 
 
 class TextRichTextWithFootnoteRef(TextRichText):
-    def __init__(self, client, notion_data, block=None, table=None):
-        super().__init__(client, notion_data, block, table)
+    def __init__(self, client, notion_data, block=None):
+        super().__init__(client, notion_data, block)
         if not self._is_footnote():
             raise UseNextClass()
 
@@ -76,7 +74,7 @@ class TextRichTextWithFootnoteRef(TextRichText):
             if ref not in self.block.page.plugin_data[plugin_data_key]:
                 pandoc_ast.append(token)
                 msg = 'Missing footnote "[%s]". Rendering as plain text (%s)'
-                logger.warning(msg, ref, self.block.notion_url)
+                self.client.logger.warning(msg, ref, self.block.notion_url)
                 continue
             self._append_footnote_to_ast(pandoc_ast, token, ref)
         return pandoc_ast
