@@ -1,17 +1,22 @@
 import re
-import logging
 from collections import deque
 
 from pandoc.types import (
-    Str, Space, LineBreak, Strong, Emph, Strikeout, Code, Link,
-    Underline, Math, InlineMath
+    Code,
+    Emph,
+    InlineMath,
+    LineBreak,
+    Link,
+    Math,
+    Space,
+    Str,
+    Strikeout,
+    Strong,
+    Underline,
 )
 
-from n2y.utils import pandoc_ast_to_markdown
 from n2y.notion_mocks import mock_rich_text
-
-
-logger = logging.getLogger(__name__)
+from n2y.utils import pandoc_write_or_log_errors
 
 
 class RichText:
@@ -24,16 +29,15 @@ class RichText:
     a property_value or somewhere else.
     """
 
-    def __init__(self, client, notion_data, block=None, table=False):
+    def __init__(self, client, notion_data, block=None):
         self.client = client
         self.block = block
-        self.table = table
 
-        self.plain_text = notion_data['plain_text']
-        self.href = notion_data.get('href', None)
-        self.notion_type = notion_data['type']
+        self.plain_text = notion_data["plain_text"]
+        self.href = notion_data.get("href", None)
+        self.notion_type = notion_data["type"]
         self.notion_data = notion_data
-        annotations = notion_data['annotations']
+        annotations = notion_data["annotations"]
         self.bold = annotations["bold"]
         self.italic = annotations["italic"]
         self.strikethrough = annotations["strikethrough"]
@@ -44,11 +48,13 @@ class RichText:
     def to_pandoc(self):
         raise NotImplementedError()
 
-    def to_markdown(self):
-        return pandoc_ast_to_markdown(self.to_pandoc()).strip('\n')
+    def to_value(self, pandoc_format, pandoc_options):
+        return pandoc_write_or_log_errors(
+            self.to_pandoc(), pandoc_format, pandoc_options, self.client.logger
+        )
 
     @classmethod
-    def plain_text_to_pandoc(klass, plain_text, table=False):
+    def plain_text_to_pandoc(cls, plain_text):
         ast = []
         match = re.findall(r"( +)|(\xa0+)|(\S+)|(\n+)|(\t+)", plain_text)
         for m in match:
@@ -60,10 +66,7 @@ class RichText:
             if word:
                 ast.append(Str(word))
             for _ in range(len(newline)):
-                if table:
-                    ast.append(Space())
-                else:
-                    ast.append(LineBreak())
+                ast.append(LineBreak())
             for _ in range(len(tab) * 4):  # 4 spaces per tab
                 ast.append(Space())
         return ast
@@ -84,9 +87,7 @@ class RichText:
 
         prependages = deque()
         appendages = deque()
-        problematic_annotations = [
-            self.bold, self.italic, self.strikethrough
-        ]
+        problematic_annotations = [self.bold, self.italic, self.strikethrough]
 
         # Notion's data model allows space to be bolded, italicized, or
         # strick through, but markdown's doesn't. Thus, since these annotations
@@ -112,53 +113,58 @@ class RichText:
 
 
 class MentionRichText(RichText):
-    def __init__(self, client, notion_data, block=None, table=False):
-        super().__init__(client, notion_data, block, table)
-        self.mention = client.wrap_notion_mention(
-            notion_data['mention'], notion_data["plain_text"], block,
-        )
+    def __init__(self, client, notion_data, block=None, mention=None):
+        super().__init__(client, notion_data, block)
+        if mention is None:
+            self.mention = client.wrap_notion_mention(
+                notion_data["mention"],
+                notion_data["plain_text"],
+                block,
+            )
+        else:
+            self.mention = mention
 
     def to_pandoc(self):
         if self.code:
-            logger.warning('Code formatting is being dropped on mention "%s"', self.plain_text)
+            self.client.logger.warning(
+                'Code formatting is being dropped on mention "%s"', self.plain_text
+            )
         mention_ast = self.mention.to_pandoc()
         return self.annotate_pandoc_ast(mention_ast)
 
 
 class EquationRichText(RichText):
-    def __init__(self, client, notion_data, block=None, table=False):
-        super().__init__(client, notion_data, block, table)
-        self.expression = notion_data['equation']['expression']
+    def __init__(self, client, notion_data, block=None):
+        super().__init__(client, notion_data, block)
+        self.expression = notion_data["equation"]["expression"]
 
     def to_pandoc(self):
         if self.code:
-            logger.warning('Code formatting is being dropped on equation "%s"', self.expression)
+            self.client.logger.warning(
+                'Code formatting is being dropped on equation "%s"', self.expression
+            )
         equation_ast = [Math(InlineMath(), self.expression)]
         return self.annotate_pandoc_ast(equation_ast)
 
 
 class TextRichText(RichText):
     @classmethod
-    def from_plain_text(klass, client, string, block=None):
+    def from_plain_text(cls, client, string, block=None):
         notion_data = mock_rich_text(string)
-        return klass(client, notion_data, block)
+        return cls(client, notion_data, block)
 
-    def __init__(self, client, notion_data, block=None, table=False):
-        super().__init__(client, notion_data, block, table)
+    def __init__(self, client, notion_data, block=None):
+        super().__init__(client, notion_data, block)
 
     def to_pandoc(self):
         if not self.code:
-            plain_text_ast = self.plain_text_to_pandoc(self.plain_text, self.table)
+            plain_text_ast = self.plain_text_to_pandoc(self.plain_text)
             annotated_ast = self.annotate_pandoc_ast(plain_text_ast)
         else:
             code_ast = [Code(("", [], []), self.plain_text)]
             annotated_ast = self.annotate_pandoc_ast(code_ast)
         if self.href:
-            return [Link(
-                ('', [], []),
-                annotated_ast,
-                (self.href, '')
-            )]
+            return [Link(("", [], []), annotated_ast, (self.href, ""))]
         else:
             return annotated_ast
 
@@ -183,13 +189,12 @@ class RichTextArray:
     be bolded, and the third would contain " the hot dog.".
     """
 
-    def __init__(self, client, notion_data, block=None, table=None):
+    def __init__(self, client, notion_data, block=None):
         self.client = client
         self.block = block
-        self.table = table
 
         assert isinstance(notion_data, list)
-        self.items = [client.wrap_notion_rich_text(i, block, table) for i in notion_data]
+        self.items = [client.wrap_notion_rich_text(i, block) for i in notion_data]
 
     def __len__(self):
         return len(self.items)
@@ -203,11 +208,13 @@ class RichTextArray:
     def to_pandoc(self):
         return sum([item.to_pandoc() for item in self.items], [])
 
-    def to_markdown(self):
-        return pandoc_ast_to_markdown(self.to_pandoc()).strip('\n')
+    def to_value(self, pandoc_format, pandoc_options):
+        return pandoc_write_or_log_errors(
+            self.to_pandoc(), pandoc_format, pandoc_options, self.client.logger
+        )
 
     def to_plain_text(self):
-        return ''.join(item.plain_text for item in self.items)
+        return "".join(item.plain_text for item in self.items)
 
     def matches(self, regexp):
         if len(self.items) > 0:
@@ -228,8 +235,5 @@ class RichTextArray:
                         self.items.pop(0)
 
     def prepend(self, string):
-        if len(self.items) > 0:
-            self.items[0].plain_text = string + self.items[0].plain_text
-        else:
-            rich_text = TextRichText.from_plain_text(self.client, string)
-            self.items.append(rich_text)
+        rich_text = TextRichText.from_plain_text(self.client, string)
+        self.items.insert(0, rich_text)
